﻿using System.Windows.Controls;
using System.Collections.ObjectModel;
using System.Windows;
using System.IO;
using System.Xml.Serialization;
using System.Runtime.Serialization;
using System;
using SaveLoadSerializationSample.DatabaseInteraction;

namespace SaveLoadSerializationSample
{
    public partial class MainPage : UserControl
    {
        DatabaseInteractionClient client;

        public MainPage()
        {
            InitializeComponent();

            client = new DatabaseInteractionClient();

            client.GetListOfViewsCompleted += client_GetListOfViewsCompleted;
            client.UploadViewCompleted += client_UploadViewCompleted;
            client.DownloadViewCompleted += client_DownloadViewCompleted;
            client.TestConnectionCompleted += client_TestConnectionCompleted;

            client.GetListOfViewsAsync();
        }
        
        void client_GetListOfViewsCompleted(object sender, GetListOfViewsCompletedEventArgs e)
        {
            this.listOfViews.ItemsSource = e.Result;
        }
        
        #region Memory Save

        SavedPivotGridView MemorySavedView;

        private void saveToMemory_Click(object sender, RoutedEventArgs e)
        {
            this.MemorySavedView = new SavedPivotGridView();
            this.MemorySavedView.SaveView(this.xamPivotGrid);
            this.lastMemorySaveTimestamp.Text = "Last save: " + this.MemorySavedView.Timestamp;
        }

        private void loadFromMemory_Click(object sender, RoutedEventArgs e)
        {
            this.MemorySavedView.Fill(this.xamPivotGrid);
        }

        #endregion Memory Save

        #region XML Serialization Save

        SavedPivotGridView XMLSerializedView;

        private void saveToXML_Click(object sender, RoutedEventArgs e)
        {
            XMLSerializedView = new SavedPivotGridView();
            XMLSerializedView.SaveView(this.xamPivotGrid);

            XmlSerializer serializer = new XmlSerializer(typeof(SavedPivotGridView));
            MemoryStream memoryStream = new MemoryStream();
            serializer.Serialize(memoryStream, XMLSerializedView);

            SaveFileDialog saveFileDialog = new SaveFileDialog()
            {
                DefaultExt = "xml",
                Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*",
                FilterIndex = 1
            };

            if (saveFileDialog.ShowDialog() == true)
            {
                using (Stream stream = saveFileDialog.OpenFile())
                {
                    byte[] bytes = memoryStream.ToArray();
                    stream.Write(bytes, 0, bytes.Length);
                    stream.Close();
                    memoryStream.Close();
                }
            }
        }

        private void loadFromXML_Click(object sender, RoutedEventArgs e)
        {
            XmlSerializer serializer = new XmlSerializer(typeof(SavedPivotGridView));

            OpenFileDialog openFileDialog = new OpenFileDialog()
            {
                Multiselect = false,
                Filter = "XML files (*.xml)|*.xml|All files (*.*)|*.*",
                FilterIndex = 1
            };

            bool? result = openFileDialog.ShowDialog();

            if (result != null && result == true)
            {
                XMLSerializedView = (SavedPivotGridView)serializer.Deserialize(openFileDialog.File.OpenRead());
                XMLSerializedView.Fill(this.xamPivotGrid);
            }
        }

        #endregion XML Serialization Save

        #region Binary Serialization Save

        private void saveViewToDB_Click(object sender, RoutedEventArgs e)
        {
            SavedPivotGridView view = new SavedPivotGridView();
            view.SaveView(this.xamPivotGrid);

            DataContractSerializer serializer = new DataContractSerializer(typeof(SavedPivotGridView));
            MemoryStream memoryStream = new MemoryStream();
            serializer.WriteObject(memoryStream, view);

            client.UploadViewAsync(DateTime.Parse(view.Timestamp), memoryStream.ToArray());
        }

        void client_UploadViewCompleted(object sender, UploadViewCompletedEventArgs e)
        {
            if (e.Result)
            {
                client.GetListOfViewsAsync();
                MessageBox.Show("Upload completed successfuly");
            }
            else
                MessageBox.Show("Upload failed");
        }

        private void loadViewFromDB_Click(object sender, RoutedEventArgs e)
        {
            if (this.listOfViews.SelectedItem != null)
                client.DownloadViewAsync(DateTime.Parse(this.listOfViews.SelectedValue.ToString(), System.Globalization.CultureInfo.InvariantCulture.DateTimeFormat));
            else
                MessageBox.Show("Please select the timestamp of the view you want to load");
        }

        void client_DownloadViewCompleted(object sender, DownloadViewCompletedEventArgs e)
        {
            if (e.Result != null)
            {
                DataContractSerializer serializer = new DataContractSerializer(typeof(SavedPivotGridView));

                int len = 0;
                while (e.Result[len] > 0)
                    len++;
                MemoryStream memoryStream = new MemoryStream(e.Result, 0, len);

                SavedPivotGridView view = serializer.ReadObject(memoryStream) as SavedPivotGridView;
                view.Fill(this.xamPivotGrid);
            }
            else
                MessageBox.Show("Upload failed");
        }

        private void testDBConnection_Click(object sender, RoutedEventArgs e)
        {
            if (client == null)
                client = new DatabaseInteractionClient();

            client.TestConnectionAsync();
        }

        void client_TestConnectionCompleted(object sender, TestConnectionCompletedEventArgs e)
        {
            if (e.Result)
                MessageBox.Show("Connection successful");
            else
                MessageBox.Show("Connection NOT successful");
        }

        #endregion Binary Serialization Save
    }
}