﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;

namespace SaveLoadSerializationSample
{
    public class SampleDataGenerator
    {
        private static String[] Products = { "Bikes", "Cars", "Trucks", "Boats" };
        private static String[] SellerNames = { "John Smith", "Alfredo Julliany", "Chris Springfield", "Dave Stevenson", "Gordon Thompson" };
        private static String[] Cities = { "New York", "San Francisco", "Miami", "Seattle", "Minneapolis" };

        private static Random random = new Random();

        public static IEnumerable GenerateSales(int numberOfSales)
        {
            List<Sale> Sales = new List<Sale>();
            for (double i = 0; i < numberOfSales; i++)
            {
                Sale sale = new Sale();
                sale.Date = GetRandomDate();
                sale.Product = GerRandomProduct();
                sale.NumberOfUnits = GetRandomNumUnits();
                sale.Seller = GetRandomSeller();
                Sales.Add(sale);
            }
            return Sales;
        }

        #region Get Random Objects
        private static Seller GetRandomSeller()
        {
            return new Seller()
            {
                City = GetRandomCity(),
                Name = GetRandomSellerName()
            };
        }

        private static string GetRandomSellerName()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return SampleDataGenerator.SellerNames[a.Next(SampleDataGenerator.SellerNames.Length)];
        }

        private static string GetRandomCity()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return SampleDataGenerator.Cities[a.Next(SampleDataGenerator.Cities.Length)];
        }

        private static int GetRandomNumUnits()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return a.Next(1, 500);
        }

        private static Product GerRandomProduct()
        {
            return new Product()
            {
                Name = GetRandomProductName(),
                Price = GetRandomPrice()
            };
        }

        private static double GetRandomPrice()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return a.NextDouble() * 100;
        }

        private static string GetRandomProductName()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            return SampleDataGenerator.Products[a.Next(SampleDataGenerator.Products.Length)];
        }

        private static DateTime GetRandomDate()
        {
            Random a = new Random(SampleDataGenerator.random.Next());
            int day = a.Next(1, 28);
            int month = a.Next(1, 12);
            int year = a.Next(2006, 2010);
            return new DateTime(year, month, day);
        }
        #endregion Get Random Objects
    }

    public class Sale
    {
        public Product Product { get; set; }
        public Seller Seller { get; set; }
        public DateTime Date { get; set; }
        public string City
        {
            get { return Seller.City; }
            set { Seller.City = value; }
        }

        [DisplayAttribute(Name = "Units sold")]
        [DisplayFormat(DataFormatString = "{0} units")]
        public int NumberOfUnits { get; set; }
        
        [DisplayAttribute(Name = "Sale amount")]
        [DisplayFormat(DataFormatString = "{0:C2}")]
        public double AmountOfSale
        {
            get { return Product.Price * NumberOfUnits; }
            set { Product.Price = value / NumberOfUnits; }
        }
    }

    public class Product
    {
        public string Name { get; set; }
        public double Price { get; set; }
    }

    public class Seller
    {
        public string Name { get; set; }
        public string City { get; set; }
    }
}
