﻿using System;
using System.ComponentModel;
using Infragistics.Olap;
using Infragistics.Olap.FlatData;

namespace SaveLoadSerializationSample
{
    public class SampleFlatDataSource : FlatDataSource
    {
        public bool Switched = false;

        public SampleFlatDataSource()
        {
            this.Cube = DataSourceBase.GenerateInitialCube("Sale");            
            this.Rows = DataSourceBase.GenerateInitialItems("[City]");
            this.Columns = DataSourceBase.GenerateInitialItems("[Product]");
            this.Filters = DataSourceBase.GenerateInitialItems("[Date]");
            this.Measures = DataSourceBase.GenerateInitialItems("AmountOfSale");

            this.ItemsSource = SampleDataGenerator.GenerateSales(500);

            CubeMetadata cubeMetadata = new CubeMetadata();
            cubeMetadata.DisplayName = "Sales Data";
            cubeMetadata.DataTypeFullName = typeof(Sale).FullName;
            this.CubesSettings.Add(cubeMetadata);

            HierarchyDescriptor<Sale> sellerHierarchy = new HierarchyDescriptor<Sale>(p => p.Seller);
            sellerHierarchy.AddLevel(p => "All sellers", "All sellers");
            sellerHierarchy.AddLevel(p => p.Seller.Name, "Individual seller");
            this.AddHierarchyDescriptor(sellerHierarchy);

            HierarchyDescriptor<Sale> productHierarchy = new HierarchyDescriptor<Sale>(p => p.Product);
            productHierarchy.AddLevel(p => "All products", "All products");
            productHierarchy.AddLevel(p => p.Product.Name, "Individual product");
            this.AddHierarchyDescriptor(productHierarchy);

            HierarchyDescriptor<Sale> locationHierarchy = new HierarchyDescriptor<Sale>(p => p.City);
            locationHierarchy.AddLevel(p => "All locations", "All locations");
            locationHierarchy.AddLevel(p => p.City, "City");
            this.AddHierarchyDescriptor(locationHierarchy);
        }

        protected override void OnResultChanged(AsyncCompletedEventArgs args)
        {
            base.OnResultChanged(args);
            if (this.Result != null && Switched == false)
                if (this.Result.ColumnAxis != null)
                    if (this.Result.ColumnAxis.Tuples.Count > 0)
                        if (this.Result.ColumnAxis.Tuples[0].Members.Count > 0)
                            if (this.Result.RowAxis != null)
                                if (this.Result.RowAxis.Tuples.Count > 0)
                                    if (this.Result.RowAxis.Tuples[0].Members.Count > 0)
                                        if (this.Result.ColumnAxis.Tuples[0].Members[0].UniqueName != String.Empty)
                                            if (this.Result.RowAxis.Tuples[0].Members[0].UniqueName != String.Empty)
                                            {
                                                this.SwitchAxisMember(this.Result.ColumnAxis.Tuples[0].Members[0]);
                                                this.SwitchAxisMember(this.Result.RowAxis.Tuples[0].Members[0]);
                                                Switched = true;
                                            }
        }
    }
}